package logging

import (
	"github.com/go-chi/chi/v5/middleware"
	"gitlab.schukai.com/oss/utilities/conan/header"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
	"net/http"
	"time"
)

var logger *zap.Logger
var sugaredLogger *zap.SugaredLogger

func InitLogger() {

	if logger != nil && sugaredLogger != nil {
		return
	}

	var mode string

	logger, _ = zap.NewDevelopment()

	defer logger.Sync() // flushes buffer, if any

	sugaredLogger = logger.Sugar()
	sugaredLogger.Infof("init %s logger", mode)

}

func LogInfo(template string, args ...interface{}) {
	if sugaredLogger == nil {
		InitLogger()
	}
	sugaredLogger.Infof(template, args)
}

func LogDebug(template string, args ...interface{}) {
	if sugaredLogger == nil {
		InitLogger()
	}
	sugaredLogger.Debugf(template, args)
}

func LogError(template string, args ...interface{}) {
	if sugaredLogger == nil {
		InitLogger()
	}
	sugaredLogger.Errorf(template, args)
}

func LoggerMiddleware(next http.Handler) http.Handler {

	fn := func(w http.ResponseWriter, r *http.Request) {
		start := time.Now()
		ww := middleware.NewWrapResponseWriter(w, r.ProtoMajor)
		next.ServeHTTP(ww, r)

		ctx := r.Context()
		requestIDString := ""
		requestID, ok := ctx.Value(header.XConanRequestID).(header.RequestUUID)
		if ok {
			requestIDString = requestID.String()
		}

		latency := time.Since(start)

		fields := []zapcore.Field{
			zap.Int("status", ww.Status()),
			zap.String("request-id", requestIDString),
			zap.Duration("took", latency),
			zap.Int64("measure.latency", latency.Nanoseconds()),
			zap.String("remote", r.RemoteAddr),
			zap.String("request", r.RequestURI),
			zap.String("method", r.Method),
		}

		if logger == nil {
			InitLogger()
		}
		logger.Info("request completed", fields...)

	}

	return http.HandlerFunc(fn)
}
