package response

import (
	"encoding/json"
	"gitlab.schukai.com/oss/utilities/conan/header"
	"gitlab.schukai.com/oss/utilities/conan/negotiator"
	"go.uber.org/zap"
	"net/http"
)

func WriteStandardResponse[D any](w http.ResponseWriter, r *http.Request, data StandardResponse[D]) {

	neg := negotiator.New(r.Header)

	if neg.Type("text/html") != "" {
		w.WriteHeader(data.Status)
	} else if neg.Type("application/json", "text/json") != "" {
		WriteStandardJson(w, data)
	} else {
		// https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/406
		w.Header().Set("X-Alvine-Hint", "Mime type not acceptable")
		w.WriteHeader(http.StatusNotAcceptable)
	}
}

type StandardResponse[D any] struct {
	Status int
	Hint   string
	Data   D
}

func WriteStandardJson[D any](w http.ResponseWriter, data StandardResponse[D]) {

	response := internalResponse{
		Sys: internalResponseSys{
			Code: data.Status,
		},
		Data: data.Data,
	}

	b, err := json.Marshal(response)
	if err != nil {
		zap.S().Error("json.Marshal fail: %s", err)
	}

	header.ApplicationJsonHeader(w)
	w.WriteHeader(data.Status)

	_, _ = w.Write(b)

}

type internalResponse struct {
	Data interface{}         `json:"data"`
	Sys  internalResponseSys `json:"sys"`
}

type internalResponseSys struct {
	Code int `json:"code"`
}
