package server

import (
	"gitlab.schukai.com/oss/utilities/conan/configuration"
	"gitlab.schukai.com/oss/utilities/conan/constants"
	"gitlab.schukai.com/oss/utilities/conan/logging"
	"net/http"
	"os"
	"regexp"
	"strings"
)

func injectHandler(next http.Handler) http.Handler {

	fn := func(w http.ResponseWriter, r *http.Request) {

		if strings.HasSuffix(r.URL.Path, ".html") {

			path := configuration.GetServerWebPath() + "/" + r.URL.Path
			p, err := os.ReadFile(path)
			if err != nil {
				logging.LogError("injectHandler error %s", err.Error())
				return
			}

			p = injectScript(p)

			w.Header().Set("Content-Type", "text/html")
			w.Write(p)

			return

		}

		next.ServeHTTP(w, r)

	}

	return http.HandlerFunc(fn)

}

var script string

func init() {

	script = `<head>
    <script type="module">

        try {
            let counter = 0;
            let socket
            let url = ""
            if (document.location.protocol == 'https:') {
                url = 'wss://'
            } else {
                url = 'ws://'
            }
            url += document.location.host + "{{ constants.WebsocketPath }}";

            function connectWebsocket() {
                
                if (socket) {
                    socket.close()
                }
                
                if (counter++ > 20) {
                    console.error("Failed 20 times to connect to websocket. Giving up.")
                    return
                }
                
                
                socket = new WebSocket(url)

                socket.onopen = function (e) {
                    console.log("[conan] Connection established");
                    counter = 0
                };

                socket.onmessage = function (event) {
                    console.log("[conan] Data received from server: " + event?.data);

                    if (event?.data == "reload") {
                        console.log("[conan] Reloading page")
                        window.location.reload();
                    }
                };

                socket.onclose = function (event) {
                    if (event.wasClean) {
                        console.log("[conan] Connection closed cleanly, code=" + event?.code + " reason=" + event?.reason + "");
                    } else {
                        console.error("[conan] Connection died");
                        setTimeout(connectWebsocket, 3000*counter)
                    }
                };

                socket.onerror = function (error) {
                    console.error("[conan] " + error?.message);
                };

            }

            document.addEventListener("DOMContentLoaded", () => {
                connectWebsocket()
            })

        } catch (e) {
            console.error(e)
        }


    </script>`

	script = strings.Replace(script, "{{ constants.WebsocketPath }}", constants.WebsocketPath, -1)

}

func injectScript(p []byte) []byte {

	s := string(p)

	reg := regexp.MustCompile(`<head>`)
	s = reg.ReplaceAllString(s, script)

	return []byte(s)

}
