package websocket

import (
	"fmt"
	"gitea.com/go-chi/session"
	"github.com/gorilla/websocket"
	"gitlab.schukai.com/oss/utilities/conan/logging"
	"log"
	"net/http"
)

var (
	upgrader    = websocket.Upgrader{}
	connections = make(map[string]*websocket.Conn)
)

func SendMessageToAll(message []byte) {

	for _, conn := range connections {
		conn.WriteMessage(websocket.TextMessage, message)
	}

}

func closeConnection(session string) error {
	if conn, ok := connections[session]; ok {
		conn.Close()
		delete(connections, session)

	}
	return nil
}

func SendReloadMessage() {
	SendMessageToAll([]byte("reload"))
}

func GetWebsocketHandler() http.Handler {

	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {

		sessionStore := session.GetSession(r)
		session := sessionStore.ID()

		conn, err := upgrader.Upgrade(w, r, nil)
		if err != nil {
			logging.LogError("websocket error %s", err.Error())
			return
		}

		conn.SetCloseHandler(func(code int, text string) error {
			return closeConnection(session)

		})

		connections[session] = conn

		defer func() {
			closeConnection(session)
		}()

		// Continuosly read and write message
		for {
			mt, message, err := conn.ReadMessage()
			if err != nil {
				log.Println("read failed:", err)
				break
			}

			if string(message) == "exit" {
				break
			}

			fmt.Println(mt, string(message), "!!!!!!!!!!!!!!!!!!")
		}
	})
}
